#include <rf60Xsdk.h>
#include <rf60Xtypes.h>

#include <iostream>

using namespace SDK;
using namespace SCANNERS;
using namespace RF60X;

int main() {
  // Initialize sdk library
  sdk_init();

  // Print return rf603 sdk version
  std::cout << sdk_version() << std::endl;

  config_base_information_rf60x_t config_params_device;

  config_params_device.type = SCANNER_TYPE::RF60x;
  config_params_device.protocol = PROTOCOLS::RIFTEK_BIN_PROTOCOL;
  config_params_device.baud_rate = BAUR_RATE_UART::Baud9600;
  config_params_device.network_address = 1;
  config_params_device.number_serial_port = "COM4";

  std::shared_ptr<rf60x> dev = std::make_shared<rf60x>(config_params_device);

  std::cout << dev->open_port_uart() << std::endl;

  dev->change_config_port_uart();

  dev->read_params_uart();

  param_t* param = dev->get_param(PARAM_NAME_KEY::USER_POWER);
  if (param->type ==
      param_value_types[(uint32_t)PARAM_VALUE_TYPE::UINT8_PARAM_TYPE]) {
    uint8_t result = param->get_value<value_uint8>();
    std::cout << " TYPE OF PROTOCOLS :" << unsigned(result) << std::endl;

    param->set_value<value_uint8>(0);
    dev->set_param(param);
    dev->write_params_uart();
  }

  uint16_t ipAddress[4];
  param = dev->get_param(PARAM_NAME_KEY::ZERO_BYTE_OF_SOURCE_IP_ADDRESS);
  if (param->type ==
      param_value_types[(uint32_t)PARAM_VALUE_TYPE::UINT8_PARAM_TYPE]) {
    ipAddress[0] = param->get_value<value_uint8>();
  }

  param = dev->get_param(PARAM_NAME_KEY::FIRST_BYTE_OF_SOURCE_IP_ADDRESS);
  if (param->type ==
      param_value_types[(uint32_t)PARAM_VALUE_TYPE::UINT8_PARAM_TYPE]) {
    ipAddress[1] = param->get_value<value_uint8>();
  }

  param = dev->get_param(PARAM_NAME_KEY::SECOND_BYTE_OF_SOURCE_IP_ADDRESS);
  if (param->type ==
      param_value_types[(uint32_t)PARAM_VALUE_TYPE::UINT8_PARAM_TYPE]) {
    ipAddress[2] = param->get_value<value_uint8>();
  }

  param = dev->get_param(PARAM_NAME_KEY::THIRD_BYTE_OF_SOURCE_IP_ADDRESS);
  if (param->type ==
      param_value_types[(uint32_t)PARAM_VALUE_TYPE::UINT8_PARAM_TYPE]) {
    ipAddress[3] = param->get_value<value_uint8>();
  }

  std::cout << "SOURCE_IP_ADDRESS :" << ipAddress[3] << "." << ipAddress[2]
            << "." << ipAddress[1] << "." << ipAddress[0] << std::endl;

  dev->close_port_uart();
}
