#include <rf60Xsdk.h>
#include <rf60Xtypes.h>

#include <iostream>

using namespace SDK;
using namespace SCANNERS;
using namespace RF60X;

int main() {
  // Initialize sdk library
  sdk_init();

  // Print return rf603 sdk version
  std::cout << sdk_version() << std::endl;

  config_base_information_rf60x_t config_params_device;

  config_params_device.type = SCANNER_TYPE::RF60x;
  config_params_device.protocol = PROTOCOLS::RIFTEK_BIN_PROTOCOL;
  config_params_device.baud_rate = BAUR_RATE_UART::Baud9600;
  config_params_device.network_address = 1;
  config_params_device.number_serial_port = "COM4";

  std::shared_ptr<rf60x> dev = std::make_shared<rf60x>(config_params_device);

  std::cout << dev->open_port_uart() << std::endl;

  dev->change_config_port_uart();

  dev->read_params_uart();

  param_t* param = dev->get_param(PARAM_NAME_KEY::PROTOCOLS_INTERFACE);
  if (param->type ==
      param_value_types[(uint32_t)PARAM_VALUE_TYPE::UINT8_PARAM_TYPE]) {
    uint8_t result = param->get_value<value_uint8>();
    std::cout << " TYPE OF PROTOCOLS :" << (unsigned)result << std::endl;

    param->set_value<value_uint8>(2);
    dev->set_param(param);
    dev->write_params_uart();
  }

  modbus_rtu_command command;
  command.address = 1;
  command.id = 1;
  command.count_or_value = 6;
  command.command = 0x04;
  uint16_t buffer[32] = {0};

  dev->send_command_modbus(command);

  dev->receive_data_modbus(buffer);

  std::cout << "Device type :" << buffer[0] << std::endl
            << "Firmware version :" << buffer[1] << std::endl
            << "serialNumber :" << buffer[2] << std::endl
            << "Base distance :" << buffer[3] << std::endl
            << "Measurement range :" << buffer[4] << std::endl
            << "Measured value :" << buffer[5] << std::endl;

  // Laser power

  command.address = 10;
  command.count_or_value = 0;  // 0 - off, 1 - on
  command.command = 0x06;
  dev->send_command_modbus(command);


  //Changing the Protocol to standard (RIFTEK)

  command.address = 39;
  command.count_or_value = 0;  // 0 - off, 1 - on
  command.command = 0x06;
  dev->send_command_modbus(command);

  dev->close_port_uart();
}
