#ifndef RF60XSDK_H
#define RF60XSDK_H

//#define EXPORTCALL __declspec(dllexport) __stdcall

#if (defined _WIN32)
#define EXPORTCALL __declspec(dllexport)
#else
#define EXPORTCALL
#endif

#ifdef __cplusplus
extern "C" {
#endif

#include <rf60X_types.h>

#include "rf60Xtypes.h"
#include "stdbool.h"

typedef struct {
  SCANNER_TYPE type;
  BAUR_RATE_UART baud_rate;
  uint8_t network_address;
  const char* number_serial_port;
  PROTOCOLS protocol;
  char* recv_ip_addrs;
} C_config_base_information_rf60x_t;

struct C_rf60x;
typedef struct C_rf60x C_rf60x_t;

/**
 * @brief version - Return info about CORE version
 * @return SDK version
 */
EXPORTCALL int version();

/**
 * @brief init - Initialize CORE library
 * Must be called once before further calls to any library functions
 * @return true if success.
 */
EXPORTCALL bool init();

/**
 * @brief cleanup - Cleanup resources allocated with core_init() function
 */

EXPORTCALL C_rf60x_t* create_rf60x(C_config_base_information_rf60x_t& config);

EXPORTCALL void destroy_r60x(C_rf60x_t* rf60x);

EXPORTCALL void cleanup();

EXPORTCALL bool connect(C_rf60x_t* rf60x);

EXPORTCALL bool disconnect(C_rf60x_t* rf60x);

EXPORTCALL bool get_measure_udp(C_rf60x_t* rf60x, void* measure);

EXPORTCALL bool open_port_uart(C_rf60x_t* rf60x);

EXPORTCALL rf60x_uart_hello_t hello_msg_uart(C_rf60x_t* rf60x);

EXPORTCALL void change_config_port_uart(C_rf60x_t* rf60x);

EXPORTCALL void close_port_uart(C_rf60x_t* rf60x);

#ifdef __cplusplus
}
#endif

#endif  // RF60XSDK_H
