#include "pch.h"

#include "rf60XsdkNet.h"
#include <msclr\marshal_cppstd.h>

rf60Xsdk::rf60XsdkNet::rf60XsdkNet(const config_base_information_rf60x_net_t^  config)
{

	config_base_information_rf60x_t *config_params_device = new config_base_information_rf60x_t();


	switch (config->type)
	{

	case SCANNER_TYPE_NET::RF60x:
		config_params_device->type = SCANNER_TYPE::RF60x;
		break;

	case SCANNER_TYPE_NET::RF60xB: 
		config_params_device->type = SCANNER_TYPE::RF60xB;
		break;

	case SCANNER_TYPE_NET::RF60xHS:
		config_params_device->type = SCANNER_TYPE::RF60xHS;
		break;
	default:
		config_params_device->type = SCANNER_TYPE::RF60x;
		break;
	}

    
	switch (config->protocol)
	{
	case PROTOCOLS_NET::RIFTEK_BIN_PROTOCOL:
		config_params_device->protocol = PROTOCOLS::RIFTEK_BIN_PROTOCOL;
		break;
		
	case PROTOCOLS_NET::MODBUS_RTU:
		config_params_device->protocol = PROTOCOLS::MODBUS_RTU;
		break;
		
	case PROTOCOLS_NET::ASCII:
		config_params_device->protocol = PROTOCOLS::ASCII;
		break;
	default:
		config_params_device->protocol = PROTOCOLS::RIFTEK_BIN_PROTOCOL;
		break;
	}


	switch (config->baud_rate)
	{
	case BAUR_RATE_UART_NET::Baud9600:
		config_params_device->baud_rate = BAUR_RATE_UART::Baud9600;
		break;

	case BAUR_RATE_UART_NET::Baud115200:
		config_params_device->baud_rate = BAUR_RATE_UART::Baud115200;
		break;

	case BAUR_RATE_UART_NET::Baud230400:
		config_params_device->baud_rate = BAUR_RATE_UART::Baud230400;
		break;
		
	case BAUR_RATE_UART_NET::Baud460800:
		config_params_device->baud_rate = BAUR_RATE_UART::Baud460800;
		break;

	case BAUR_RATE_UART_NET::Baud921600:
		config_params_device->baud_rate = BAUR_RATE_UART::Baud921600;
		break;
	default:
		config_params_device->baud_rate = BAUR_RATE_UART::Baud9600;
		break;
	}

	String^ localValueSerialPort = config->serial_port_number;
	config_params_device->network_address = config->network_address;
	config_params_device->number_serial_port = marshal_as<std::string>(localValueSerialPort);

	IntPtr ptrToNativeString = Marshal::StringToHGlobalAnsi(config->recv_ip_addrs);

	config_params_device->recv_ip_addrs = static_cast<char*>(ptrToNativeString.ToPointer());



	_rf60x = new SDK::SCANNERS::RF60X::rf60x(*config_params_device);

	
}


bool rf60Xsdk::rf60XsdkNet::connect()
{
	return  _rf60x->connect();
}


bool rf60Xsdk::rf60XsdkNet::get_measure_udp(udp_measure_net_t% clone)
{ 

	udp_measure_t measure;
	
	bool result = 0;

	result = _rf60x->get_measure_udp(&measure);

	if (!result)
		return false;

	clone.deviceBaseDistance = measure.deviceBaseDistance;
	clone.deviceMeasureRange = measure.deviceMeasureRange;
	clone.deviceSerial = measure.deviceSerial;
	clone.packCount = measure.packCount;
	clone.packetControlSumm = measure.packetControlSumm;

	for (Byte i = 0; i < 168; ++i) {
		clone.Array[i].status = measure.rf60xValArray[i].status;
		clone.Array[i].value = measure.rf60xValArray[i].value;

	}

	return result;



}

bool rf60Xsdk::rf60XsdkNet::disconnect()
{
	return  _rf60x->disconnect();
}

bool rf60Xsdk::rf60XsdkNet::open_port_uart()
{
	return _rf60x->open_port_uart();
}

rf60Xsdk::uart_hello_rf60x_net_t rf60Xsdk::rf60XsdkNet::hello_msg_uart()

{
	uart_hello_t hello_packet = _rf60x->hello_msg_uart();
 
	uart_hello_rf60x_net_t  hello_packet_net;

	hello_packet_net.deviceMaxDistance = hello_packet.deviceMaxDistance;
	hello_packet_net.deviceModificaton = hello_packet.deviceModificaton;
	hello_packet_net.deviceSerial = hello_packet.deviceSerial;
	hello_packet_net.deviceType = hello_packet.deviceType;
	hello_packet_net.deviceRange= hello_packet.deviceRange;

	return hello_packet_net;

}

void rf60Xsdk::rf60XsdkNet::change_config_port_uart()
{
	_rf60x->change_config_port_uart();
}

bool rf60Xsdk::rf60XsdkNet::send_command(COMMAND_UART_NET CMD)

{
	COMMAND_UART innerCmd;
	switch (CMD)
	{
	case rf60Xsdk::COMMAND_UART_NET::WRITEPARAM:
		innerCmd = COMMAND_UART::WRITEPARAM;
		break;
	case rf60Xsdk::COMMAND_UART_NET::SAVETOFLASH:
		innerCmd = COMMAND_UART::SAVETOFLASH;
		break;
	case rf60Xsdk::COMMAND_UART_NET::RESTOREFORMFLASH:
		innerCmd = COMMAND_UART::RESTOREFORMFLASH;
		break;
	case rf60Xsdk::COMMAND_UART_NET::START_STREAM:
		innerCmd = COMMAND_UART::START_STREAM;
		break;
	case rf60Xsdk::COMMAND_UART_NET::STOP_STREAM:
		innerCmd = COMMAND_UART::STOP_STREAM;
		break;
	default:
		break;
	}

	return _rf60x->send_command(innerCmd);
}

void rf60Xsdk::rf60XsdkNet::close_port_uart()
{
	_rf60x->close_port_uart();
}

 bool rf60Xsdk::rf60XsdkNet::get_measure_uart(uart_stream_measure_net_t% measure,PROTOCOL_MEASURE_UART_NET type)
{
	bool result;



	 uart_stream_measure_t innerMeasure;

		result = _rf60x->get_measure_uart(&innerMeasure,PROTOCOL_MEASURE_UART::UART_STREAM_MEASURE_T);

	
		if (!result)
			return result;

		measure.status = innerMeasure.status;
		measure.value = innerMeasure.value;
	
		return result;
}
 
 bool rf60Xsdk::rf60XsdkNet::get_measure_uart(uart_result_with_encoder_net_t % measure, PROTOCOL_MEASURE_UART_NET type)
 {
	 bool result;

	 uart_result_with_encoder_t innerMeasure;


	 result = _rf60x->get_measure_uart(&innerMeasure, PROTOCOL_MEASURE_UART::UART_RESULT_WITH_ENCODER_T);


		 if (!result)
			 return false;

		 measure.encoderValue = innerMeasure.encoderValue;
		 measure.value = innerMeasure.measure;
		 measure.rotationMarks = innerMeasure.rotationMarks;

	 return result;
 }

 bool rf60Xsdk::rf60XsdkNet::send_command_modbus(modbus_rtu_command_net_t% command)
 {
	 modbus_rtu_command innerCommand;
	 pin_ptr<modbus_rtu_command_net_t> pCommand = &command;
	 memcpy_s(&innerCommand, sizeof(modbus_rtu_command), pCommand, sizeof(modbus_rtu_command_net_t));

	 return _rf60x->send_command_modbus(innerCommand);
 }

 bool rf60Xsdk::rf60XsdkNet::receive_data_modbus(array<System::UInt16>^ buffer)
 {
	 buffer->Clear;
	 pin_ptr<unsigned short> pBuffer = &buffer[0];

	 return _rf60x->receive_data_modbus(pBuffer);
 }

 bool rf60Xsdk::rf60XsdkNet::read_params_uart()
 {
	 return _rf60x->read_params_uart();
 }

 byte rf60Xsdk::rf60XsdkNet::get_param(PARAM_NAME_KEY_NET param)
 {

	 valueParam = nullptr;

	 PARAM_NAME_KEY innerParams;
     
	 switch (param) {

	 case PARAM_NAME_KEY_NET::USER_POWER:
		 innerParams = PARAM_NAME_KEY::USER_POWER;
		 break;
	 case PARAM_NAME_KEY_NET::POWER_ANALOG_OUTPUT:
		 innerParams = PARAM_NAME_KEY::POWER_ANALOG_OUTPUT;
			 break;
	 case PARAM_NAME_KEY_NET::CONTROL_OF_AVERAGING:
		 innerParams = PARAM_NAME_KEY::CONTROL_OF_AVERAGING;
			 break;
	 case PARAM_NAME_KEY_NET::NETWORK_ADDRESS_UART:
		 innerParams = PARAM_NAME_KEY::NETWORK_ADDRESS_UART;
			 break;
	 case PARAM_NAME_KEY_NET::BAUD_RATE_UART:
		 innerParams = PARAM_NAME_KEY::BAUD_RATE_UART;
			 break;
	 case PARAM_NAME_KEY_NET::NUMBER_OF_AVERAGED_VALUES:
		 innerParams = PARAM_NAME_KEY::NUMBER_OF_AVERAGED_VALUES;
			 break;
	 case PARAM_NAME_KEY_NET::LOW_BYTE_OF_THE_SAMPLING_PERIOD:
		 innerParams = PARAM_NAME_KEY::LOW_BYTE_OF_THE_SAMPLING_PERIOD;
			 break;
	 case PARAM_NAME_KEY_NET::HIGH_BYTE_OF_THE_SAMPLING_PERIOD:
		 innerParams = PARAM_NAME_KEY::HIGH_BYTE_OF_THE_SAMPLING_PERIOD;
			 break;
	 case PARAM_NAME_KEY_NET::LOW_BYTE_OF_MAX_INTEGRATION_TIME:
		 innerParams = PARAM_NAME_KEY::LOW_BYTE_OF_MAX_INTEGRATION_TIME;
			 break;
	 case PARAM_NAME_KEY_NET::HIGH_BYTE_OF_MAX_INTEGRATION_TIME:
		 innerParams = PARAM_NAME_KEY::HIGH_BYTE_OF_MAX_INTEGRATION_TIME;
			 break;
	 case PARAM_NAME_KEY_NET::LOW_BYTE_FOR_THE_BEGINNING_OF_ANALOG_OUT_RANGE:
		 innerParams = PARAM_NAME_KEY::LOW_BYTE_FOR_THE_BEGINNING_OF_ANALOG_OUT_RANGE;
			 break;
	 case PARAM_NAME_KEY_NET::HIGH_BYTE_FOR_THE_BEGINNING_OF_ANALOG_OUT_RANGE:
		 innerParams = PARAM_NAME_KEY::HIGH_BYTE_FOR_THE_BEGINNING_OF_ANALOG_OUT_RANGE;
			 break;
	 case PARAM_NAME_KEY_NET::LOW_BYTE_FOR_THE_END_OF_ANALOG_OUT_RANGE:
		 innerParams = PARAM_NAME_KEY::LOW_BYTE_FOR_THE_END_OF_ANALOG_OUT_RANGE;
			 break;
	 case PARAM_NAME_KEY_NET::HIGH_BYTE_FOR_THE_END_OF_ANALOG_OUT_RANGE:
		 innerParams = PARAM_NAME_KEY::HIGH_BYTE_FOR_THE_END_OF_ANALOG_OUT_RANGE;
			 break;
	 case PARAM_NAME_KEY_NET::TIME_LOCK_OF_RESULT:
		 innerParams = PARAM_NAME_KEY::TIME_LOCK_OF_RESULT;
			 break;
	 case PARAM_NAME_KEY_NET::LOW_BYTE_OF_A_ZERO_POINT:
		 innerParams = PARAM_NAME_KEY::LOW_BYTE_OF_A_ZERO_POINT;
			 break;
	 case PARAM_NAME_KEY_NET::HIGH_BYTE_OF_A_ZERO_POINT:
		 innerParams = PARAM_NAME_KEY::HIGH_BYTE_OF_A_ZERO_POINT;
			 break;
	 case PARAM_NAME_KEY_NET::DATA_TRANSFER_RATE_VIA_CAN_INTERFACE:
		 innerParams = PARAM_NAME_KEY::DATA_TRANSFER_RATE_VIA_CAN_INTERFACE;
			 break;
	 case PARAM_NAME_KEY_NET::LOW_BYTE_OF_STANDARD_IDENTIFIER:
		 innerParams = PARAM_NAME_KEY::LOW_BYTE_OF_STANDARD_IDENTIFIER;
			 break;
	 case PARAM_NAME_KEY_NET::HIGH_BYTE_OF_STANDARD_IDENTIFIER:
		 innerParams = PARAM_NAME_KEY::HIGH_BYTE_OF_STANDARD_IDENTIFIER;
			 break;
	 case PARAM_NAME_KEY_NET::ZERO_BYTE_OF_EXTENDED_IDENTIFIER:
		 innerParams = PARAM_NAME_KEY::ZERO_BYTE_OF_EXTENDED_IDENTIFIER;
			 break;
	 case PARAM_NAME_KEY_NET::FIRST_BYTE_OF_EXTENDED_IDENTIFIE:
		 innerParams = PARAM_NAME_KEY::FIRST_BYTE_OF_EXTENDED_IDENTIFIE;
			 break;
	 case PARAM_NAME_KEY_NET::SECOND_BYTE_OF_EXTENDED_IDENTIFIER:
		 innerParams = PARAM_NAME_KEY::SECOND_BYTE_OF_EXTENDED_IDENTIFIER;
			 break;
	 case PARAM_NAME_KEY_NET::THIRD_BYTE_OF_EXTENDED_IDENTIFIER:
		 innerParams = PARAM_NAME_KEY::THIRD_BYTE_OF_EXTENDED_IDENTIFIER;
			 break;
	 case PARAM_NAME_KEY_NET::CAN_INTERFACE_IDENTIFIER:
		 innerParams = PARAM_NAME_KEY::CAN_INTERFACE_IDENTIFIER;
			 break;
	 case PARAM_NAME_KEY_NET::CAN_INTERFACE_POWER_SWITCH:
		 innerParams = PARAM_NAME_KEY::CAN_INTERFACE_POWER_SWITCH;
			 break;
	 case PARAM_NAME_KEY_NET::ZERO_BYTE_OF_DESTINATION_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::ZERO_BYTE_OF_DESTINATION_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::FIRST_BYTE_OF_DESTINATION_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::FIRST_BYTE_OF_DESTINATION_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::SECOND_BYTE_OF_DESTINATION_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::SECOND_BYTE_OF_DESTINATION_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::THIRD_BYTE_OF_DESTINATION_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::THIRD_BYTE_OF_DESTINATION_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::ZERO_BYTE_OF_GATEWAY_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::ZERO_BYTE_OF_GATEWAY_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::FIRST_BYTE_OF_GATEWAY_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::FIRST_BYTE_OF_GATEWAY_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::SECOND_BYTE_OF_GATEWAY_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::SECOND_BYTE_OF_GATEWAY_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::THIRD_BYTE_OF_GATEWAY_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::THIRD_BYTE_OF_GATEWAY_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::ZERO_BYTE_OF_SUBNET_MASK:
		 innerParams = PARAM_NAME_KEY::ZERO_BYTE_OF_SUBNET_MASK;
			 break;
	 case PARAM_NAME_KEY_NET::FIRST_BYTE_OF_SUBNET_MASK:
		 innerParams = PARAM_NAME_KEY::FIRST_BYTE_OF_SUBNET_MASK;
			 break;
	 case PARAM_NAME_KEY_NET::SECOND_BYTE_OF_SUBNET_MASK:
		 innerParams = PARAM_NAME_KEY::SECOND_BYTE_OF_SUBNET_MASK;
			 break;
	 case PARAM_NAME_KEY_NET::THIRD_BYTE_OF_SUBNET_MASK:
		 innerParams = PARAM_NAME_KEY::THIRD_BYTE_OF_SUBNET_MASK;
			 break;
	 case PARAM_NAME_KEY_NET::ZERO_BYTE_OF_SOURCE_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::ZERO_BYTE_OF_SOURCE_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::FIRST_BYTE_OF_SOURCE_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::FIRST_BYTE_OF_SOURCE_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::SECOND_BYTE_OF_SOURCE_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::SECOND_BYTE_OF_SOURCE_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::THIRD_BYTE_OF_SOURCE_IP_ADDRESS:
		 innerParams = PARAM_NAME_KEY::THIRD_BYTE_OF_SOURCE_IP_ADDRESS;
			 break;
	 case PARAM_NAME_KEY_NET::LOW_BYTE_OF_THE_NUMBER_OF_MEASUREMENTS_IN_THE_PACKET:
		 innerParams = PARAM_NAME_KEY::LOW_BYTE_OF_THE_NUMBER_OF_MEASUREMENTS_IN_THE_PACKET;
			 break;
	 case PARAM_NAME_KEY_NET::HIGH_BYTE_OF_THE_NUMBER_OF_MEASUREMENTS_IN_THE_PACKET:
		 innerParams = PARAM_NAME_KEY::HIGH_BYTE_OF_THE_NUMBER_OF_MEASUREMENTS_IN_THE_PACKET;
			 break;
	 case PARAM_NAME_KEY_NET::ETH_INTERFACE_ON_OFF:
		 innerParams = PARAM_NAME_KEY::ETH_INTERFACE_ON_OFF;
			 break;
	 case PARAM_NAME_KEY_NET::AUTO_OF_THE_STREAM:
		 innerParams = PARAM_NAME_KEY::AUTO_OF_THE_STREAM;
			 break;
	 case PARAM_NAME_KEY_NET::PROTOCOLS_INTERFACE:
		 innerParams = PARAM_NAME_KEY::PROTOCOLS_INTERFACE;
			 break;
	}

	valueParam = _rf60x->get_param(innerParams);
	 if (valueParam->type ==
		 param_value_types[(uint32_t)PARAM_VALUE_TYPE::UINT8_PARAM_TYPE]) {
		 return valueParam->get_value<value_uint8>();
	 }



	 return -1;


 }


 bool rf60Xsdk::rf60XsdkNet::set_param(byte value) {
	 valueParam->set_value<value_uint8>(value);
	 if (!_rf60x->set_param(valueParam))
		 return FALSE;

	 return TRUE;
 }

 bool rf60Xsdk::rf60XsdkNet::write_params_uart()
 {
	 return _rf60x->write_params_uart();
 }

rf60Xsdk::rf60XsdkNet::~rf60XsdkNet()
{
	delete _rf60x;
}

bool rf60Xsdk::rf60XsdkNet::sdk_init()
{
	return SDK::SCANNERS::RF60X::sdk_init();
}

void rf60Xsdk::rf60XsdkNet::sdk_cleanup()
{
	SDK::SCANNERS::RF60X::sdk_cleanup();
}


int rf60Xsdk::rf60XsdkNet::sdk_version()
{
	return SDK::SCANNERS::RF60X::sdk_version();
}
