#include "rf60x.h"

#include <rf60Xsdk.h>
#include <rf60Xtypes.h>
#include <stdlib.h>

using namespace SDK;
using namespace SCANNERS;
using namespace RF60X;

struct rf60x_c {
  void *obj;
};

rf60x_t *rf60x_create(
    const namespace_struct_wrapper::config_base_information_rf60x_t
        config_params_device) {
  if (!sdk_init()) return NULL;

  config_base_information_rf60x_t inner_config_params_device;

  rf60x_t *m;
  rf60x *obj;

  switch (config_params_device.baud_rate) {
    case namespace_struct_wrapper::BAUR_RATE_UART::Baud9600: {
      inner_config_params_device.baud_rate = BAUR_RATE_UART::Baud9600;
      break;
    }
    case namespace_struct_wrapper::BAUR_RATE_UART::Baud230400: {
      inner_config_params_device.baud_rate = BAUR_RATE_UART::Baud230400;
      break;
    }
    case namespace_struct_wrapper::BAUR_RATE_UART::Baud460800: {
      inner_config_params_device.baud_rate = BAUR_RATE_UART::Baud460800;
      break;
    }
    case namespace_struct_wrapper::BAUR_RATE_UART::Baud115200: {
      inner_config_params_device.baud_rate = BAUR_RATE_UART::Baud115200;
      break;
    }
    case namespace_struct_wrapper::BAUR_RATE_UART::Baud921600: {
      inner_config_params_device.baud_rate = BAUR_RATE_UART::Baud921600;
      break;
    }
  }

  inner_config_params_device.network_address =
      config_params_device.network_address;
  inner_config_params_device.number_serial_port =
      config_params_device.number_serial_port;

  m = (decltype(m))malloc(sizeof(*m));
  obj = new rf60x(inner_config_params_device);
  m->obj = obj;

  return m;
}

void rf60x_destroy(rf60x_t *m) {
  if (m == NULL) return;
  delete static_cast<rf60x *>(m->obj);
  free(m);
  sdk_cleanup();
}

bool rf60x_connect(rf60x_t *dev) {
  return static_cast<rf60x *>(dev->obj)->connect();
}

bool rf60x_disconnect(rf60x_t *dev) {
  return static_cast<rf60x *>(dev->obj)->disconnect();
}

bool rf60x_get_measure_udp(rf60x_t *dev, void *measure) {
  return static_cast<rf60x *>(dev->obj)->get_measure_udp(measure);
}

bool rf60x_open_port_uart(rf60x_t *dev) {
  return static_cast<rf60x *>(dev->obj)->open_port_uart();
}

namespace_struct_wrapper::uart_hello_t rf60x_hello_msg_uart(rf60x_t *dev) {
  auto uartHello = static_cast<rf60x *>(dev->obj)->hello_msg_uart();

  namespace_struct_wrapper::uart_hello_t wrapperUartHello;

  wrapperUartHello.deviceMaxDistance = uartHello.deviceMaxDistance;
  wrapperUartHello.deviceModificaton = uartHello.deviceModificaton;
  wrapperUartHello.deviceRange = uartHello.deviceRange;
  wrapperUartHello.deviceSerial = uartHello.deviceSerial;
  wrapperUartHello.deviceType = uartHello.deviceType;

  return wrapperUartHello;
}

void rf60x_change_config_port_uart(rf60x_t *dev) {
  static_cast<rf60x *>(dev->obj)->open_port_uart();
}

void rf60x_close_port_uart(rf60x_t *dev) {
  static_cast<rf60x *>(dev->obj)->close_port_uart();
}

bool rf60x_get_measure_uart(
    rf60x_t *dev, void *measure,
    namespace_struct_wrapper::PROTOCOL_MEASURE_UART type) {
  switch (type) {
    case namespace_struct_wrapper::PROTOCOL_MEASURE_UART::UART_STREAM_MEASURE_T:
      return static_cast<rf60x *>(dev->obj)->get_measure_uart(
          measure, PROTOCOL_MEASURE_UART::UART_STREAM_MEASURE_T);
      break;
    case namespace_struct_wrapper::PROTOCOL_MEASURE_UART::
        UART_STREAM_ADVANCED_MEASURE_T:
      return static_cast<rf60x *>(dev->obj)->get_measure_uart(
          measure, PROTOCOL_MEASURE_UART::UART_STREAM_ADVANCED_MEASURE_T);
      break;
    case namespace_struct_wrapper::PROTOCOL_MEASURE_UART::
        UART_STREAM_MODIFIED_MEASURE_T:
      return static_cast<rf60x *>(dev->obj)->get_measure_uart(
          measure, PROTOCOL_MEASURE_UART::UART_STREAM_MODIFIED_MEASURE_T);
      break;

    case namespace_struct_wrapper::PROTOCOL_MEASURE_UART::
        UART_RESULT_WITH_ENCODER_T:
      return static_cast<rf60x *>(dev->obj)->get_measure_uart(
          measure, PROTOCOL_MEASURE_UART::UART_RESULT_WITH_ENCODER_T);
      break;
  }
}

bool rf60x_get_single_measure(rf60x_t *dev, unsigned char address,
                              void *measure) {
  return static_cast<rf60x *>(dev->obj)->get_single_measure(address, measure);
}

bool rf60x_send_command_modbus(
    rf60x_t *dev, const namespace_struct_wrapper::modbus_rtu_command &command) {
  modbus_rtu_command innerCommand;

  innerCommand.id = command.id;

  innerCommand.command = command.command;

  innerCommand.address = command.address;

  innerCommand.count_or_value = command.count_or_value;

  innerCommand.crc = command.crc;

  return static_cast<rf60x *>(dev->obj)->send_command_modbus(innerCommand);
}

bool rf60x_receive_data_modbus(rf60x_t *dev, unsigned short *buffer) {
  return static_cast<rf60x *>(dev->obj)->receive_data_modbus(buffer);
}

bool rf60x_read_params_uart(rf60x_t *dev) {
  return static_cast<rf60x *>(dev->obj)->read_params_uart();
}

bool rf60x_write_params_uart(rf60x_t *dev) {
  return static_cast<rf60x *>(dev->obj)->write_params_uart();
}

bool rf60x_send_command(rf60x_t *dev,
                        namespace_struct_wrapper::COMMAND_UART CMD) {
  switch (CMD) {
    case namespace_struct_wrapper::COMMAND_UART::GETRESULT:
      return static_cast<rf60x *>(dev->obj)->send_command(
          COMMAND_UART::GETRESULT);
      break;
    case namespace_struct_wrapper::COMMAND_UART::SAVETOFLASH:
      return static_cast<rf60x *>(dev->obj)->send_command(
          COMMAND_UART::SAVETOFLASH);
      break;
    case namespace_struct_wrapper::COMMAND_UART::START_STREAM:
      return static_cast<rf60x *>(dev->obj)->send_command(
          COMMAND_UART::START_STREAM);
      break;
    case namespace_struct_wrapper::COMMAND_UART::START_STREAM_ADVANCED:
      return static_cast<rf60x *>(dev->obj)->send_command(
          COMMAND_UART::START_STREAM_ADVANCED);
      break;

    case namespace_struct_wrapper::COMMAND_UART::START_STREAM_MODIFIED:
      return static_cast<rf60x *>(dev->obj)->send_command(
          COMMAND_UART::START_STREAM_MODIFIED);
      break;

    case namespace_struct_wrapper::COMMAND_UART::STOP_STREAM:
      return static_cast<rf60x *>(dev->obj)->send_command(
          COMMAND_UART::STOP_STREAM);
      break;

    case namespace_struct_wrapper::COMMAND_UART::WRITEPARAM:
      return static_cast<rf60x *>(dev->obj)->send_command(
          COMMAND_UART::WRITEPARAM);
      break;
  }
}

// param_t *rf60x_get_param(rf60x_t *dev, char *param_name) {}

// param_t *rf60x_get_param(rf60x_t *dev, PARAM_NAME_KEY param_name) {}

// bool rf60x_set_param(rf60x_t *dev, param_t *param) {}

// bool rf60x_set_param(rf60x_t *dev, const char *param_name) {}

// bool rf60x_set_param(rf60x_t *dev, int param_id) {}

bool r60x_restore_params(rf60x_t *dev) {
  return static_cast<rf60x *>(dev->obj)->send_command(
      COMMAND_UART::RESTOREFORMFLASH);
}
